/**
 * Solution Verifier
 * 
 * Author: Gabriel Corvera
 * Cornell University / ITESM CEM
 * 
 */
 
  #include <stdio.h>
  #include <errno.h>
  #include <stdlib.h>
  #include <string.h>
  #include "solutionver.h"
  
  int offset = 0;
  
  /**
   * readCNF. This function returns true if the file was successfully loaded into the data structures
   */
  boolean readCNF(char *filename){
 	FILE *f;	
 	FILE *out;
 	int i,j,k,temp,t_array[MAX_CLAUSE_VARS];
 	char c,s[200];
 	clause *temporalClause;
 	/*if the file did not open I should return false*/
 	if ((f = fopen (filename, "r")) == NULL) {
    	printf("\nc readCNF - ERROR OPENING FILE '%s'", filename); 
    	return false; 
    }
 	if ((out = fopen ("out.cnf", "w")) == NULL) {
    	printf("\nc readCNF - ERROR OPENING FILE '%s'", filename); 
    	return false; 
    }    
    /*Begin reading comments in the input file*/
 	fscanf(f, "%c", &c);
  	while(c == 'c') {
    	do
      		fscanf(f, "%c", &c);
    	while(c != '\n');
    	fscanf(f, "%c", &c);
  	}    
  	/*if we are not reading comments we should be in the p line*/
  	if(c != 'p') { 
  		printf("\nc readCNF - ERROR OPENING FILE '%s'", filename); 
  		return false; 
  	}
  	fseek(f, (long) (-1), SEEK_CUR);
  	fgets(s, 6, f);
  	/*we verify if the file is in cnf format.*/
  	if(strcmp("p cnf",s) || (feof(f))) { 
  		printf("\nc readCNF - BAD FILE FORMAT '%s'", filename); 
  		return false; 
  	}
  	/*The file shouldn't finish here*/
  	if(feof(f)) {
    	printf("\nc readCNF - BAD FILE FORMAT '%s'", filename);
   		return false;
  	}
  	fscanf(f, "%d%d", &numVars, &numClauses);
  	fprintf(out,"p cnf %d %d",numVars,numClauses);
  	/* assing enough space for an int array for the variables*/
  	variables = (int *)malloc(sizeof(int)*(numVars+1));
  	minsat = (int *)malloc(sizeof(int)*(numVars+1));
  	/* initialize with ceros */
 	for(i=0;i<numVars+1;i++){
		variables[i]=0;
		minsat[i]=0;
 	}  	
  	
  	/*time to read the clauses and fill the datastructures*/
  	for(i=0;i<numClauses;i++){
		if(feof(f)){
			printf("\nc readCNF - INCORRECT NUMBER OF CLAUSES '%s'", filename); 
			return false;
		}
		j = 0;
		/*prepare space to store clause*/
		temporalClause = (clause *)malloc(sizeof(clause));
		temporalClause->id=i+1;
		do{
		/*read the integer of the formula*/
	      	fscanf(f, "%d", &temp);
	      	if(temp!=0){
		      	t_array[j++]=((temp>0)?temp-offset:temp+offset);
	      	}else{
	      		t_array[j++]=0;
	      	}
		}while(temp!=0);
		t_array[j]=0;
		for(k=0;k<j;k++){
			temporalClause->vars[k]=t_array[k];
			fprintf(out,"%d ",t_array[k]);
		}
		fprintf(out,"\n");
		temporalClause->varsSize=j-1;
		/*assign the next clause*/
		temporalClause->next=clauses;
		clauses=temporalClause;
		
  	}	
  	return true;	
 }

 int findNumberOfSATClauses(){
 	int i,sat_clauses=0,var;
 	clause *cl_pointer; 	
 	cl_pointer=clauses;
 	
 	/*search every clause to see how many clauses are satisfied by the actual 
 	 variable assignation*/
 	while(cl_pointer!=NULL){
	 	for(i=0;i<cl_pointer->varsSize;i++){
	 		var = cl_pointer->vars[i];
 			if(var>0){

 				/*the variable var is true! then the clause is sat 	*/			
 				if(variables[var]){
 					sat_clauses++;
 					break;
 				} 				
 			}else if(var<0){
				/*the variable var is false! then the clause is sat */
 				if(!variables[-var]){
 					sat_clauses++;
 					break;
 				} 				 				
 			}
 			
 		}
 		cl_pointer=cl_pointer->next;
 	}	
 	return sat_clauses;
 }
 
  /**
   * Insert solution to program
   */
 
  void readTruthAssignment(){
 	int i,temp;
 	printf("Introduce the values of each variable as a binary string separeted by spaces (i.e. x -y z => 1 0 1) [%d variables]: \n",numVars);
 	for(i=1;i<numVars+1;i++){
 		if(i<numVars){
			scanf("%d ",&temp);
 		}else{
			scanf("%d",&temp); 			
 		}
		variables[i]=temp;
 	} 	
 }
 
 
 
 int findMINSAT(){
 	long i,sat_clauses,min=numClauses;

 	readTruthAssignment();	
	sat_clauses = findNumberOfSATClauses();
	
 	return sat_clauses;
 }

 int main (int argc,char * args[]){
 	int j;
 	if(argc>3 || argc<2){
 		printf("USAGE: %s filename.cnf [offset]\n",args[0]); 
 		exit(0);
 	}
 	if(argc==3){
 		offset=atoi(args[2]);
 	}
 	if(!readCNF(args[1])){
 		return false;
 	}	 	 	

	printf("c The given formula with the given solution has: %d SAT clauses.\n",findMINSAT());
 	return true;
 }
